'PURPOSE: Play embedded wav or mid files.

' This file has in it a Module named Global that allows a variable to be
'used between NameSpaces, Functions and Subs.
'It also has two Namespaces named myWav and myMid
'Each of the Namespaces contain a Class each with Functions and/or Subs.

'Assume you have an Embedded Midi File named playMid
' and/or an Embedded Wave File named playWav, then the Calls would be:
'    myMIDI.WinMid.PlayMIDIResource("playMid.mid")
'    myWav.WinWav.PlayWavResource("playWav.wav")
'    myMIDI.WinMid.killTemp()...... This will remove the temporary mid file upon closing application.
'    myMIDI.WinMid.StopMidi()
Imports System.Resources
Imports System.IO
Module Global
    'Allow use between NameSpaces, Functions and Subs
    Public TempMidiFilePath As String
End Module
Namespace myWav

    Public Class WinWav
        'Declare constants 
        Public Const SND_ASYNC As Int32 = 1
        Public Const SND_MEMORY As Int32 = 4

        'this is the API we want to play embedded Wav resource... 
        Public Declare Function PlaySound Lib "winmm.dll" Alias "PlaySound" (ByVal data() As Byte, ByVal hMod As IntPtr, ByVal dwFlags As Int32) As Boolean

        Public Shared Function PlayWavResource(ByVal EmbeddedWav As String)
            'BE SURE (embedded).wav HAS BUILD ACTION IN PROPERTIES SET TO EMBEDDED RESOURCE
            'get the namespace 
            Dim strNameSpace As String = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name.ToString()
            ' get the resource into a stream 
            Dim Str As Stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(strNameSpace + "." + EmbeddedWav)
            If Str Is Nothing Then
                MsgBox("Unable to find: " & EmbeddedWav & vbCrLf & "Be Sure " & EmbeddedWav & " Property Build Action is set to Embedded Resource" _
                & vbCrLf & "Another reason can be that the Project Root Namespace is not the same as the Assembly Name")
                Exit Function
            End If
            If Str.CanRead Then
                'bring stream into a byte array 
                Dim bStr(Str.Length) As Byte
                Str.Read(bStr, 0, Int(Str.Length))
                'play the resource 
                PlaySound(bStr, IntPtr.Zero, SND_ASYNC Or SND_MEMORY)
            End If
        End Function

    End Class

End Namespace

Namespace myMIDI
    Public Class WinMid
        Private Declare Function mciSendString Lib "winmm.dll" Alias "mciSendStringA" (ByVal lpstrCommand As String, ByVal lpstrReturnString As String, ByVal uReturnLength As Long, ByVal hwndCallback As Long) As Long
        'Use Windows Temp Path for temp.mid
        'VB6 -Private Declare Function GetTempPath Lib "kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
        'VB.Net is different...Long is not acceptable, Integer is used instead.
        Private Declare Function GetTempPath Lib "kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Integer, ByVal lpBuffer As String) As Integer
        Private Const MAX_PATH = 260


        Public Shared Function PlayMIDIResource(ByVal EmbeddedMid As String)
            'Get the Windows Temp folder location

            'get the user's windows\temp folder
            Dim sWinTmpDir As String
            'pad the passed string
            sWinTmpDir = Space$(MAX_PATH)

            'get the folder
            Dim r As Integer
            r = GetTempPath(MAX_PATH, sWinTmpDir)

            'r contains the number of chrs up to the
            'terminating null, so a simple Microsoft.VisualBasic.Left can
            'be used. It's also conveniently terminated
            'with a slash.
            'In a class that has a Left property, it may be necessary to fully qualify the Left function.
            'Microsoft.VisualBasic.Left must be used!!!
            sWinTmpDir = Microsoft.VisualBasic.Left(sWinTmpDir, r)
            TempMidiFilePath = sWinTmpDir & "temp.mid"
            If Dir$(TempMidiFilePath) <> "" Then Kill(TempMidiFilePath)

            'BE SURE (embedded).mid HAS BUILD ACTION IN PROPERTIES SET TO EMBEDDED RESOURCE
            mciSendString("stop midi", "", 0, 0)
            mciSendString("close midi", "", 0, 0)
            'get the namespace 
            Dim strNameSpace As String = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name.ToString()
            ' get the resource into a stream 
            Dim Str As Stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(strNameSpace + "." + EmbeddedMid)
            If Str Is Nothing Then
                MsgBox("Unable to find: " & EmbeddedMid & vbCrLf & "Be Sure " & EmbeddedMid & " Property Build Action is set to Embedded Resource" _
                & vbCrLf & "Another reason can be that the Project Root Namespace is not the same as the Assembly Name")
                Exit Function
            End If
            If Str.CanRead Then
                'bring stream into a byte array 
                Dim bStr(Str.Length) As Byte
                Str.Read(bStr, 0, Int(Str.Length))
                'We now have a bStr array with the (embedded).mid in it.
                'Write to file so we can use mciSendString
                'There should be a way to use the bStr direct, but I can not figure how
                '====Write Binary temp.mid file
                Dim i As Integer

                'Requires Imports System.IO
                Dim objBinaryWriter As BinaryWriter
                Dim FS As FileStream
                'creating temp.mid to store file 
                FS = New FileStream(TempMidiFilePath, FileMode.OpenOrCreate)
                'Pass the file path and the file name to the BinaryWriter constructor.
                objBinaryWriter = New BinaryWriter(FS)
                For i = 0 To UBound(bStr) - 1
                    'Write binary data
                    objBinaryWriter.Write(bStr(i))
                Next
                'Close the file.
                objBinaryWriter.Close()
                'PLAY midi resource
                Dim lRet As Long
                On Error Resume Next
                lRet = mciSendString("stop midi", "", 0, 0)
                lRet = mciSendString("close midi", "", 0, 0)
                lRet = mciSendString("open sequencer!" & TempMidiFilePath & " alias midi", "", 0, 0)
                lRet = mciSendString("play midi", "", 0, 0)
                PlayMIDIResource = (lRet = 0)
            End If
        End Function
        Public Shared Sub killTemp()
            mciSendString("stop midi", "", 0, 0)
            mciSendString("close midi", "", 0, 0)
            'Clean Up
            If TempMidiFilePath = "" Then Exit Sub
            Try
                Kill(TempMidiFilePath)
            Catch ex As FileNotFoundException
                Exit Sub
            Catch ex As Exception
                MsgBox("Exception in killTemp: " & ex.Message)
            End Try

        End Sub
        Public Shared Sub StopMidi()
            mciSendString("stop midi", "", 0, 0)
            mciSendString("close midi", "", 0, 0)
        End Sub
    End Class
End Namespace
